@@ -415,17 +415,22 @@ class Sherlock:
             logging.info('teff = %.6f', star_info.teff)
             logging.info('lum = %.6f', star_info.lum)
             logging.info('logg = %.6f', star_info.logg)
-        star_df = pandas.DataFrame(columns=['ra', 'dec', 'R_star', 'R_star_lerr', 'R_star_uerr', 'M_star',
+        star_df = pandas.DataFrame(columns=['obj_id', 'ra', 'dec', 'R_star', 'R_star_lerr', 'R_star_uerr', 'M_star',
                                             'M_star_lerr', 'M_star_uerr', 'Teff_star', 'Teff_star_lerr',
                                             'Teff_star_uerr', 'ld_a', 'ld_b'])
-        star_df = star_df.append({'ra': star_info.ra, 'dec': star_info.dec, 'R_star': star_info.radius,
+        star_df = star_df.append({'obj_id': object_info.mission_id(), 'ra': star_info.ra, 'dec': star_info.dec, 'R_star': star_info.radius,
                                   'R_star_lerr': star_info.radius - star_info.radius_min,
                         'R_star_uerr': star_info.radius_max - star_info.radius,
                         'M_star': star_info.mass, 'M_star_lerr': star_info.mass - star_info.mass_min,
                         'M_star_uerr': star_info.mass_max - star_info.mass,
                         'Teff_star': star_info.teff, 'Teff_star_lerr': 200, 'Teff_star_uerr': 200,
-                                  'logg': star_info.logg,
-                                  'ld_a': star_info.ld_coefficients[0], 'ld_b': star_info.ld_coefficients[1]},
+                                  'logg': star_info.logg, 'logg_err': star_info.logg_err,
+                                  'ld_a': star_info.ld_coefficients[0], 'ld_b': star_info.ld_coefficients[1],
+                                  'feh': star_info.feh,
+                                  'feh_err': star_info.feh_err, 'j': star_info.j, 'j_err': star_info.j_err,
+                                  'k': star_info.k, 'k_err': star_info.k_err,
+                                  'h': star_info.h, 'h_err': star_info.h_err,
+                                  'kp': star_info.kp},
                        ignore_index=True)
         star_df.to_csv(object_dir + "params_star.csv", index=False)
         logging.info('================================================')
